import 'package:chessboard_js/chessboard/board_buttons.dart';
import 'package:chessboard_js/chessboard_notation/chessboard_notation.dart';
import 'package:chessboard_js/utils/utils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/painting.dart';
import 'square_ui.dart';
import '../images/resizable_draggable.dart';
import 'controller.dart';
import '../utils/constants.dart';

/// Chessboard Widget, receives size and a chessboard controller
class ChessBoard extends StatefulWidget {
  ChessBoard({
    required this.initialSize,
    required this.controller,
    this.minSize = 300,
    this.boardType = BoardType.brown,
    this.notationPanelBackgroundColor = Colors.black,
    this.notationPanelWidth = 200,
    this.notationColor = Colors.white,
    this.showNotationPanel = true,
  })  : assert(
          initialSize >= minSize,
          'size property must be greater or equal to minSize',
        ),
        assert(notationPanelWidth >= 200,
            'notationPanelWidth must be at least 200');

  final ChessboardController controller;
  final double minSize;
  final double initialSize;
  final BoardType boardType;
  final Color notationPanelBackgroundColor;
  final double notationPanelWidth;
  final Color notationColor;
  final bool showNotationPanel;

  @override
  _ChessBoardState createState() => _ChessBoardState();
}

class _ChessBoardState extends State<ChessBoard> {
  late double size;
  late bool whiteTowardUser;
  @override
  void initState() {
    size = widget.initialSize;
    whiteTowardUser = widget.controller.whiteTowardUser;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    /// Make sure the board size wont get messed up
    final screenSize = MediaQuery.of(context).size;
    final smallestSide = screenSize.width < screenSize.height
        ? screenSize.width
        : screenSize.height;
    if (size > smallestSide) {
      size = smallestSide;
    }
    if (widget.initialSize < widget.minSize) size = widget.minSize;

    return MakeMoveShortcut(
      enabled: widget.controller.keyboardEnabled,
      onGoBack: () => widget.controller.goBack(),
      onGoForward: () => widget.controller.goForward(),
      child: Container(
        child: Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            Container(
              width: size,
              height: size,
              child: Stack(
                children: <Widget>[
                  Align(
                    alignment: Alignment.topCenter,
                    child: Container(
                      width: size * 0.90,
                      height: size * 0.90,
                      child: getBoardImage(widget.boardType),
                    ),
                  ),

                  /// the pieces on the chessboard
                  /// meaning, state of each square
                  Align(
                    alignment: Alignment.topCenter,
                    child: FittedBox(
                      fit: BoxFit.contain,
                      child: Container(
                        width: size * 0.90,
                        height: size * 0.90,
                        child: buildChessBoard(),
                      ),
                    ),
                  ),
                  Positioned(
                      right: size * 0.025,
                      bottom: size * 0.075,

                      /// To resize the chessboard. New size must be
                      /// less than the smallest screen side and
                      /// greater than the min size variable
                      child: ResizableDraggable(
                        size: size / 20,
                        onDrag: (double offset) {
                          if (size + offset >= widget.minSize &&
                              size + offset <= smallestSide)
                            setState(() {
                              size = size + offset * 1.5;
                            });
                        },
                      )),
                  if ((screenSize.width < size + 150) ||
                      (!widget.showNotationPanel))
                    Align(
                      alignment: Alignment.bottomCenter,
                      child: Padding(
                        padding: const EdgeInsets.only(top: 4.0),
                        child: BoardButtons(
                          width: size * 0.9,
                          onResign: widget.controller.onResign ?? null,
                          flipBoard: () =>
                              setState(() => widget.controller.flipBoard()),
                        ),
                      ),
                    ),
                ],
              ),
            ),
            if (screenSize.width > size + 150)
              Container(
                height: size,
                width: 150,
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.start,
                  children: [
                    /// the notation for the chessboard
                    if (widget.showNotationPanel)
                      ChessboardNotation(
                        height: size * 0.9 - 60,
                        width: 150,
                        controller: widget.controller,
                        notationTextColor: Colors.white,
//                        onTapMove: (tappedMove) {
//                          print('tapped move');
//                        widget.controller.setCurrentMove(tappedMove);
//                        },
                      ),
                    if (widget.showNotationPanel)
                      BoardButtons(
                          width: 150,
                          onResign: widget.controller.onResign,
                          flipBoard: () =>
                              setState(() => widget.controller.flipBoard())),
                  ],
                ),
              ),
          ],
        ),
      ),
    );
  }

  /// Builds the board, drawing each square
  Widget buildChessBoard() {
    final pieceSize = size / 8;
    return Column(
      children: widget.controller.whiteTowardUser
          ? squareList.map((row) {
              return Expanded(
                flex: 1,
                child: Row(
                  children: row
                      .map((squareName) => BoardSquare(
                            key: ValueKey(squareName),
                            squareName: squareName,
                            size: pieceSize,
                            controller: widget.controller,
                            onMove: (move) {
                              widget.controller.makePrettyMove(move);
                            },
                          ))
                      .toList(),
                ),
              );
            }).toList()
          : squareList.reversed.map((row) {
              return Expanded(
                flex: 1,
                child: Row(
                  children: row.reversed
                      .map((squareName) => BoardSquare(
                            key: ValueKey(squareName),
                            squareName: squareName,
                            size: widget.initialSize,
                            controller: widget.controller,
                            onMove: (move) {
                              widget.controller.makePrettyMove(move);
                            },
                          ))
                      .toList(),
                ),
              );
            }).toList(),
    );
  }

  /// Returns the board image
  Image? getBoardImage(BoardType boardType) {
    switch (boardType) {
      case BoardType.brown:
        return Image.asset(
          'packages/chessboard_js/images/brown_board.png',
          fit: BoxFit.contain,
        );
      case BoardType.darkBrown:
        return Image.asset(
          "packages/chessboard_js/images/dark_brown_board.png",
          fit: BoxFit.cover,
        );
      case BoardType.green:
        return Image.asset(
          "packages/chessboard_js/images/green_board.png",
          fit: BoxFit.cover,
        );
      case BoardType.orange:
        return Image.asset(
          "packages/chessboard_js/images/orange_board.png",
          fit: BoxFit.cover,
        );
      default:
        return null;
    }
  }
}
